// Predict retweet activity using TiDeH.
#include "TiDeH.h" 

// Solve Integral Equation for activity: 
void  solv_sc_lambda( int max, double t_st, struct data_rt data, double ave_n, double par[], double s0, double lam[] );

// Num. of Event between t ¥in [t_1, t_2]
int  cnt_event( int t_1, int t_2, int num, struct data_rt data);
// Calculate mean number of follwers:
double  calc_ave_n( struct data_rt data );
// Calc. p(t) (Infactious rate)
double  calc_pT( double t, double par[] );


int main(int argc, char *argv[])
{ 
  if( argc != 7 )   
    {   printf("Usage:./a.out *f_in *f_par *f_pred $t_st (h) $s0 (h) $w_bin (h)\n");  exit(1); }
  
  FILE *f_in, *f_pred, *f_par; 
  f_in = fopen( argv[1],  "r");
  if ( f_in == NULL)    {  printf("Cannot open $f_in (pred) \n");  exit(1);   }
  f_par = fopen( argv[2], "r");
  if ( f_par == NULL)   {  printf("Cannot open $f_par\n");  exit(1);  }
  f_pred = fopen( argv[3], "w");
  if ( f_pred == NULL)   {  printf("Cannot open $f_pred\n");  exit(1);  }    
  
  int     i, j, num;  
  double  par[6], t_0, t_en= 168, t_st= atof(argv[4]);
  struct data_rt  data;  
  
  fscanf( f_in, "%d %lf", &num, &t_0);     // Read #Event 
  data.n_i = calloc( num, sizeof(int) );       data.num= 0;
  data.t_sp= calloc( num, sizeof(double) );
  
  for (i=0; i<num+1; i++)   // Read event time and #follwers
    {  
      fscanf( f_in, "%lf %d ", &data.t_sp[i], &data.n_i[i]);          
      if ( data.t_sp[i]< t_st )   data.num++;  
    }
  // Average Num. of Degree 
  double   ave_n= calc_ave_n(data);

  // Read parameters
  int      max= (t_en-t_st)/DT;     
  double   p_amp, s0= atof(argv[5]), *lam;     
  // p_0, phi_0 (lag), r, k_m
  fscanf( f_par, "%lf %lf %lf %lf ", &par[0], &par[1], &par[2], &par[3] );
  par[2] += t_0;
  
  // Limit parameter range: Non-Bursting Condition. 
  p_amp= par[0]* (1+ fabs(par[1]) );
  if ( ave_n* p_amp > 1 )     ave_n= 1.0/ p_amp;
  
  lam= calloc( max, sizeof(double) );   
  
  // Predict future activity based on TiDeH
  solv_sc_lambda( max, t_st, data, ave_n, par, s0, lam );  
  
  int      n_bin, n_dat, n_p;
  double   err= 0, sum, w_bin= atof(argv[6]);
  n_bin= (int)(w_bin/DT);     n_p= (int)( (t_en- t_st)/w_bin); 
  
  for (i=0; i< n_p; i++)   // Show Activity
    {   
      n_dat= cnt_event( t_st+i*w_bin, t_st+(i+1)*w_bin, num, data );
      
      sum=   0;
      for (j=0; j<n_bin; j++)     sum += lam[i*n_bin+j]; 
      sum=  sum/n_bin;
      
      err +=  fabs( n_dat- w_bin* sum); 
      fprintf( f_pred, "%f %d %f\n", t_st+i*w_bin, n_dat, w_bin* sum );    
    }
    
  printf(  "%f \n", err );   // Write Errs 
  
  free(data.t_sp);      free(data.n_i);    free(lam); 
  fclose(f_in);         fclose(f_pred);     fclose(f_par);
  return 0;        
}


void  solv_sc_lambda( int max, double t_st, struct data_rt data, double ave_n, double par[], double s0, double lam[] )	
{
  int      i, j;
  double   alpha_t, c_t, c0, conv, p_T, t_i;
  
  c0= 1.0/s0/ (1- 1.0/Th);     t_i= t_st/24.0;  
  p_T= calc_pT( t_i, par );    // t_i: (day)
  lam[0]= p_T* calc_gt( t_st, s0, data );  // t_st (hour)
  
  for (i=1; i<max; i++)
    { 
      t_i= (t_st+ i*DT)/24.0;   // Time (Day)
      p_T= calc_pT( t_i, par );    
      
      alpha_t= ave_n* p_T* DT;    
      c_t= 1.0/ (1- alpha_t* c0/2.0);  
      
      conv= calc_phi( i*DT, c0, s0)* lam[0]/2.0;
      for (j=1; j<i; j++)  // Calc. lambda
	{   conv += calc_phi( (i-j)*DT, c0, s0)* lam[j];   } 
      
      lam[i]= c_t* ( p_T* calc_gt( t_st+i*DT, s0, data)+ alpha_t*conv );
    }
}


int  cnt_event( int t_1, int t_2, int num, struct data_rt data)	
{
  int k1= 0, k2= 0;
  
  while ( data.t_sp[k1] < t_2 && k1< num+1 ) 
    {          
      if ( data.t_sp[k1] < t_1 )   k2++;
      k1++;
    }
  return  k1-k2;
}


double  calc_pT( double t, double par[] )
{
  double  p_T, theta= 2* M_PI* (t+ par[2]);
  
  p_T= par[0]* (1- par[1]* sin(theta)  )* exp( (-1)*t/par[3] );  
  return   p_T;    
}


double  calc_ave_n( struct data_rt data )
{    // Calculate mean number of follwers:
  int i;     double sum= 0;    
  if ( data.num == 1 )   {  return  0;  }    
  else
    {
      for (i=1; i< data.num; i++)     //  Check up to data.num-1
	{  sum += data.n_i[i];   }   //    sum: ¥sum ¥n_i[i]      

      return  sum/(data.num-1); 
    }
}
